/***************************************************************************/
/***************************************************************************/
/*                                                                         */
/*   (c) 1995.  The Regents of the University of California.  All rights   */
/*   reserved.                                                             */
/*                                                                         */
/*   This work was produced at the University of California, Lawrence      */
/*   Livermore National Laboratory (UC LLNL) under contract no.            */
/*   W-7405-ENG-48 (Contract 48) between the U.S. Department of Energy     */
/*   (DOE) and The Regents of the University of California (University)    */
/*   for the operation of UC LLNL.  Copyright is reserved to the           */
/*   University for purposes of controlled dissemination,                  */
/*   commercialization through formal licensing, or other disposition      */
/*   under terms of Contract 48; DOE policies, regulations and orders;     */
/*   and U.S. statutes.  The rights of the Federal Government are          */
/*   reserved under Contract 48 subject to the restrictions agreed upon    */
/*   by the DOE and University.                                            */
/*                                                                         */
/*                                                                         */
/*                              DISCLAIMER                                 */
/*                                                                         */
/*   This software was prepared as an account of work sponsored by an      */
/*   agency of the United States Government.  Neither the United States    */
/*   Government nor the University of California nor any of their          */
/*   employees, makes any warranty, express or implied, or assumes any     */
/*   liability or responsibility for the accuracy, completeness, or        */
/*   usefulness of any information, apparatus, product, or process         */
/*   disclosed, or represents that its specific commercial products,       */
/*   process, or service by trade name, trademark, manufacturer, or        */
/*   otherwise, does not necessarily constitute or imply its               */
/*   endorsement, recommendation, or favoring by the United States         */
/*   Government or the University of California. The views and opinions    */
/*   of the authors expressed herein do not necessarily state or reflect   */
/*   those of the United States Government or the University of            */
/*   California, and shall not be used for advertising or product          */
/*   endorsement purposes.                                                 */
/*                                                                         */
/*   Permission to use, copy, modify and distribute this software and its  */
/*   documentation for any non-commercial purpose, without fee, is         */
/*   hereby granted, provided that the above copyright notice and this     */
/*   permission notice appear in all copies of the software and            */
/*   supporting documentation, and that all UC LLNL identification in      */
/*   the user interface remain unchanged.  The title to copyright LLNL     */
/*   XDIR shall at all times remain with The Regents of the University     */
/*   of California and users agree to preserve same. Users seeking the     */
/*   right to make derivative works with LLNL XDIR for commercial          */
/*   purposes may obtain a license from the Lawrence Livermore National    */
/*   Laboratory's Technology Transfer Office, P.O. Box 808, L-795,         */
/*   Livermore, CA 94550.                                                  */
/*                                                                         */
/***************************************************************************/
/***************************************************************************/

#include <Xm/SelectioB.h>
#include <Xm/LabelG.h>
#include <Xm/Form.h>
#include "xdir.h"
#include "list.h"
#include "str.h"

#define MAXLEVELS  20

struct {
    int recursive;                      /* Requires initialization */
    int level;                          /* Requires initialization */
    struct dirwin_st *dirwin;           /* Requires initialization */
    struct entry_link *head[MAXLEVELS]; /* Requires initialization */
	int error;							/* Requires initialization */
	int asked_user;						/* Requires initialization */
    char *rel_path;
    char *full_path;
} del_ctrl;

static char *del_question = "You are attempting to delete a (possibly nonempty)\ndirectory, but the general preference DELETE MODE\nis set to NONRECURSIVE.\n\nDelete nonempty directories anyway?";

static char *msg1 = "Unable to redisplay directory.\n\nDirectory window might be out of date.";

extern struct st_host_info hinfo[];
extern int beep_when_ops_done;
extern Widget w_toplev;
extern int diagnostics;
extern int inquire_on_delete;
extern XtAppContext app;
extern int rdelete_mode;

char *merge_paths();
char *links_to_path();
int cb_delete_entries();
char **path_to_links();


/*
 * cb_delete - Callback that deletes selected file(s).
 */
void
cb_delete(widget, client_data, call_data)
Widget widget;
XtPointer client_data;
XtPointer call_data;
{
	struct dirwin_st *dirwin = (struct dirwin_st *)client_data;
    int i;
    struct entry_link *head;
	struct entry_link *tail;

    /* Clear error flag */
    raise_okflag();

    /* Ask the user if okay to delete? */
    if (inquire_on_delete) {
		if (!verify_selection(dirwin, dirwin,
                "Do you really want to delete these entries?"))
            return;
    }

    /* This might take some time */
    use_busy_cursor();

    /* Pop up deletion monitor */
    show_mp_monitor(dirwin, "Now Deleting:");

    /* Disable controls */
    enable_controls(False);

	/* Form list of selected items to delete */
	head = NULL;
	tail = NULL;
    for (i=0; i<dirwin->nentries; i++)
        if (dirwin->entries[i].level == 0)
            postorder_selected_items(&dirwin->entries[i], &head, &tail);

    /* Set up deletion control block */
    del_ctrl.recursive = rdelete_mode;
    del_ctrl.level = 0;
    del_ctrl.dirwin = dirwin;
    del_ctrl.head[0] = head;
	del_ctrl.error = False;
	del_ctrl.asked_user = False;

    /* Perform deletion */
    XtAppAddWorkProc(app, (XtWorkProc)cb_delete_entries, NULL);
}


/*
 * clean_up_del_ctrl - Cleans up the deletion control block data structure.
 *                     This routine should be called after servicing a
 *                     deletion request. 
 */
clean_up_del_ctrl()
{
	int i;

	XtFree(del_ctrl.rel_path);
	del_ctrl.rel_path = NULL;

	XtFree(del_ctrl.full_path);
	del_ctrl.full_path = NULL;

	for(i=0; i<=del_ctrl.level; i++)
		release_linked_list(&del_ctrl.head[i]);
}


/*
 * cb_delete_entries - Work procedure for executing deletion request.
 *                     State information is kept in "del_ctrl", the
 *                     deletion control block data structure.
 */
cb_delete_entries()
{
	static char *del_error_msg =
		"Unable to delete entry(s).  Can't get directory list.";
	struct entry_link *ptr;
	struct dirwin_st *dirwin = del_ctrl.dirwin;
	int host = dirwin->host;
	struct sl_struct *list;
	int retval;
	int len;
	int i;
	int is_a_directory;
	char msg[MAXPATHLEN+40];
	int already_handled_dir = False;
	char **rel_path_links;
	char *temp;
	char *q;
	int nlinks;
	char **link_ptr;
	int indx;
	char **first_path_links;
	char **p;

	del_ctrl.full_path = NULL;
	del_ctrl.rel_path = NULL;
		
	/* Did user push abort button? */
	if (mp_abort_requested())
		goto done;

	if (del_ctrl.head[del_ctrl.level] == NULL) {
		if (del_ctrl.level == 0) {
			if (del_ctrl.error)
				record_and_alert("At least one deletion failed.",
					del_ctrl.dirwin->w_shell);
			goto done;
		} else {
			del_ctrl.level--;
			already_handled_dir = True;
		}
	}

	/* Build relative path name */
	first_path_links = path_to_links(hinfo[host].system,
		del_ctrl.head[0]->entry);
	nlinks = 0;
	link_ptr = first_path_links;
	while (*link_ptr++)
		nlinks++;
	nlinks += del_ctrl.level;
	rel_path_links = (char **)XtMalloc(sizeof(char *)*(nlinks+1));
	indx = 0;
	p = first_path_links;
	while (*p) {
		rel_path_links[indx++] = XtNewString(*p);
		p++;
	}
	release_path_links(first_path_links);
	for (i=1; i<=del_ctrl.level; i++)
		rel_path_links[indx++] = XtNewString(del_ctrl.head[i]->entry);
	rel_path_links[nlinks] = NULL;
	del_ctrl.rel_path = links_to_path(hinfo[host].system, rel_path_links,
		nlinks);
	release_path_links(rel_path_links);

	/* Build full path name */
	del_ctrl.full_path = merge_paths(hinfo[host].system, dirwin->dirname,
		del_ctrl.rel_path);

	/* Handle recursion */
	if (!already_handled_dir) {
		if (del_ctrl.recursive || !del_ctrl.asked_user) {
			/* Test for directory by trying to cd to it */
			is_a_directory = False;
			if (host == LOCAL) {
				if (local_cd(del_ctrl.full_path, False) == 0)
					is_a_directory = True;
			} else
				switch (remote_cd(host, del_ctrl.full_path, True, False)) {
				case -3:
					goto lost;
				case 0:
					is_a_directory = True;
				}
			if (is_a_directory) {
				if (!del_ctrl.recursive) {
					beep();
					if (question_dialog(del_question, dirwin->w_shell))
						del_ctrl.recursive = True;
					del_ctrl.asked_user = True;
				}
				if (del_ctrl.recursive) {
					/* Get directory list */
					if (host == LOCAL) {
						if (local_ls(del_ctrl.full_path, &list, False, True)) {
							record_and_alert(del_error_msg, 
								del_ctrl.dirwin->w_shell);
							goto done;
						}
					} else {
						switch (remote_ls(host, del_ctrl.full_path, &list,
							False, True)) {
						case -3:
							goto lost;
						case -1:
							record_and_alert(del_error_msg,
								del_ctrl.dirwin->w_shell);
							goto done;
						}
					}
					/* Add directory list to new level */
					del_ctrl.level++;
					if (del_ctrl.level == MAXLEVELS) {
						record_and_alert(
							"Deletion(s) aborted.  Recursion too deep.",
							del_ctrl.dirwin->w_shell);
						goto done;
					}
					del_ctrl.head[del_ctrl.level] = NULL;
					for (i=list->nentries-1; i>=0; i--) {
						ptr = XtNew(struct entry_link);
						ptr->entry = XtNewString(list->entries[i]);
						ptr->next = del_ctrl.head[del_ctrl.level];
						del_ctrl.head[del_ctrl.level] = ptr;
					}
					release_array_list(list);
				    XtFree(del_ctrl.full_path);
				    XtFree(del_ctrl.rel_path);
					return False;
				}
			}
		}
	}

	/* Delete next directory entry */
	update_mp_monitor(del_ctrl.rel_path);
	if (host == LOCAL)
		retval = local_delete(del_ctrl.full_path);
	else {
		len = strlen(del_ctrl.full_path);
		if ((hinfo[host].system == SYS_VMS)
				&& (del_ctrl.full_path[len-1] == ']'))
			if (hinfo[host].server == SERVER_VMS_3_0) {    /* Kludge */
				temp = XtMalloc(strlen(del_ctrl.full_path)+8);
				strcpy(temp, del_ctrl.full_path);
				if ((q = strrchr(temp, '.')) == NULL)
					fatal_error("Trouble in cb_delete_entries()");
				*q = '\0';
				strcat(temp, "]");
				strcat(temp, del_ctrl.head[del_ctrl.level]->entry);
				retval = remote_rmdir(host, temp, True);
				XtFree(temp);
			} else
				retval = remote_rmdir(host, del_ctrl.full_path, True);
		else if (hinfo[host].server == SERVER_MAC_FTPD)
			switch (remote_cd(host, del_ctrl.full_path, True, False)) {
			case -3:
				goto lost;
			case 0:
				retval = remote_rmdir(host, del_ctrl.full_path, True);
				break;
			default:
				retval = remote_delete(host, del_ctrl.full_path, True);
			}
		else
			retval = remote_delete(host, del_ctrl.full_path, True);
	}
    switch (retval) {
    case -3:
		goto lost;
    case -1:
        sprintf(msg, "Unable to delete %s.", del_ctrl.rel_path);
        record_warning(msg);
		del_ctrl.error = True;
		break;
	case 0:
		if (diagnostics >= NORMAL) {
        	sprintf(msg, "*** Successfully deleted:  %s\n", del_ctrl.rel_path);
        	write_log(msg);
		}
    }
    XtFree(del_ctrl.full_path);
    XtFree(del_ctrl.rel_path);
    ptr = del_ctrl.head[del_ctrl.level];
    del_ctrl.head[del_ctrl.level] = ptr->next;
    XtFree(ptr->entry);
    XtFree((char *)ptr);
    return False;

done:
	
	clean_up_del_ctrl();
	if (mp_abort_requested()) {
		write_log("*** Deletion(s) aborted.");
		hide_abort_dialog();
	} else
		hide_mp_monitor();
	enable_controls(True);
	switch (display_dir(host, dirwin, dirwin->dirname, True, False)) {
	case -3:
		lost_connection(host, dirwin);
		break;
	case -1:
		record_and_alert(msg1, dirwin->w_shell);
	}
	update_dir_controls();
	if (beep_when_ops_done)
		beep();
	restore_prev_cursor();
	return True;

lost:

	clean_up_del_ctrl();
	if (mp_abort_requested()) {
		write_log("*** Deletion(s) aborted.");
		hide_abort_dialog();
	} else
		hide_mp_monitor();
	enable_controls(True);
	clear_selected_entries();
	restore_prev_cursor();
	lost_connection(host, dirwin);
	use_busy_cursor();
	update_dir_controls();
	if (beep_when_ops_done)
		beep();
	restore_prev_cursor();
	return True;
}

