/***************************************************************************/
/***************************************************************************/
/*                                                                         */
/*   (c) 1995.  The Regents of the University of California.  All rights   */
/*   reserved.                                                             */
/*                                                                         */
/*   This work was produced at the University of California, Lawrence      */
/*   Livermore National Laboratory (UC LLNL) under contract no.            */
/*   W-7405-ENG-48 (Contract 48) between the U.S. Department of Energy     */
/*   (DOE) and The Regents of the University of California (University)    */
/*   for the operation of UC LLNL.  Copyright is reserved to the           */
/*   University for purposes of controlled dissemination,                  */
/*   commercialization through formal licensing, or other disposition      */
/*   under terms of Contract 48; DOE policies, regulations and orders;     */
/*   and U.S. statutes.  The rights of the Federal Government are          */
/*   reserved under Contract 48 subject to the restrictions agreed upon    */
/*   by the DOE and University.                                            */
/*                                                                         */
/*                                                                         */
/*                              DISCLAIMER                                 */
/*                                                                         */
/*   This software was prepared as an account of work sponsored by an      */
/*   agency of the United States Government.  Neither the United States    */
/*   Government nor the University of California nor any of their          */
/*   employees, makes any warranty, express or implied, or assumes any     */
/*   liability or responsibility for the accuracy, completeness, or        */
/*   usefulness of any information, apparatus, product, or process         */
/*   disclosed, or represents that its specific commercial products,       */
/*   process, or service by trade name, trademark, manufacturer, or        */
/*   otherwise, does not necessarily constitute or imply its               */
/*   endorsement, recommendation, or favoring by the United States         */
/*   Government or the University of California. The views and opinions    */
/*   of the authors expressed herein do not necessarily state or reflect   */
/*   those of the United States Government or the University of            */
/*   California, and shall not be used for advertising or product          */
/*   endorsement purposes.                                                 */
/*                                                                         */
/*   Permission to use, copy, modify and distribute this software and its  */
/*   documentation for any non-commercial purpose, without fee, is         */
/*   hereby granted, provided that the above copyright notice and this     */
/*   permission notice appear in all copies of the software and            */
/*   supporting documentation, and that all UC LLNL identification in      */
/*   the user interface remain unchanged.  The title to copyright LLNL     */
/*   XDIR shall at all times remain with The Regents of the University     */
/*   of California and users agree to preserve same. Users seeking the     */
/*   right to make derivative works with LLNL XDIR for commercial          */
/*   purposes may obtain a license from the Lawrence Livermore National    */
/*   Laboratory's Technology Transfer Office, P.O. Box 808, L-795,         */
/*   Livermore, CA 94550.                                                  */
/*                                                                         */
/***************************************************************************/
/***************************************************************************/

#include <Xm/PushBG.h>
#include <Xm/SelectioB.h>
#include <Xm/Text.h>
#include "xdir.h"
#include "list.h"

static Widget w_gotoDirDialog;

static char *help_goto_dir[] = {
    "This dialog enables you to display the directory specified",
    "in the \"Directory\" field.\n",
    "\n",
    "Previously referenced directories are cached and displayed",
    "in the scrollable list labeled \"Recently Referenced",
    "Directories\".  Double-click on a path in the list to",
    "display that directory.  Single-click on a path in the",
    "list to place that entry into the text field labeled",
    "\"Directory\", where it can be edited.\n",
    "\n",
    "The host's directory cache is modified every time one of",
    "its directories is displayed, regardless of the",
    "mechanism used to specify a directory to display.  The user",
    "preference SORT CACHES specifies whether the cached",
    "directory paths are displayed in alphabetical order or",
    "with the most-recently-referenced paths at the top of",
    "the list.  The user preference DIRECTORY CACHE SIZE",
    "specifies the maximum number of directory paths to cache",
    "per host.  The directory cache is preserved across LLNL XDIR",
    "sessions.\n",
    "\n",
    "Click on the OK button to perform the operation and",
    "remove the dialog.  Click on the CANCEL button to remove",
    "the dialog without performing the operation.",
    NULL
};

extern struct dirwin_st *dirwin_head;
extern struct st_host_info hinfo[];
extern int sort_caches;
extern int auto_refresh;

void cb_map_dialog();
void cb_goto_dir_ok();
void cb_goto_dir_cancel();
void cb_goto_dir_help();
Widget vertical_sb();
char **path_to_links();
char *links_to_path();
char *cstring_to_text();


/*
 * cb_goto_dir - Display the "Go to Directory" dialog.
 */
void
cb_goto_dir(widget, client_data, call_data)
Widget widget;
XtPointer client_data;
XtPointer call_data;
{
	struct dirwin_st *dirwin = (struct dirwin_st *)client_data;
	Widget w_dir;

    /* Clear error flag */
    raise_okflag();

    create_goto_dir_dialog(dirwin);

    update_goto_dir_dialog(dirwin);

    XtManageChild(w_gotoDirDialog);
	w_dir = XmSelectionBoxGetChild(w_gotoDirDialog, XmDIALOG_TEXT);
	traverse_to_widget(w_dir);

	/* Make help dialog accessible */
	make_help_modal();
}


/*
 * create_goto_dir_dialog - Create "Go to Directory" dialog.  "dirwin" is
 *                          the directory window to center the "Go To"
 *                          dialog over.
 */
create_goto_dir_dialog(dirwin)
struct dirwin_st *dirwin;
{
	Arg args[2];
	int i;
	Widget w_list;

	i = 0;
	XtSetArg(args[i], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); i++;
	XtSetArg(args[i], XmNdefaultPosition, False); i++;
	w_gotoDirDialog = XmCreateSelectionDialog( dirwin->w_shell, "gotoDir",
		args,i);
	XtAddCallback(w_gotoDirDialog, XmNokCallback, cb_goto_dir_ok,
		(XtPointer)dirwin);
	XtAddCallback(w_gotoDirDialog, XmNcancelCallback, cb_goto_dir_cancel, NULL);
	XtAddCallback(w_gotoDirDialog, XmNhelpCallback, cb_goto_dir_help, NULL);
	XtAddCallback(w_gotoDirDialog, XmNmapCallback, cb_map_dialog,
		(XtPointer)XtWindow(dirwin->w_shell));

	/* Kludge to add title bar under OLWM */
	AddOLWMDialogFrame(w_gotoDirDialog);

	/* Enhance appearance of scrolled list widget */
	w_list = XmSelectionBoxGetChild(w_gotoDirDialog, XmDIALOG_LIST);

	/* Don't need "Apply" button */
	XtUnmanageChild(XmSelectionBoxGetChild(w_gotoDirDialog,
		XmDIALOG_APPLY_BUTTON));
}


/*
 * update_goto_dir_dialog - Display most-recently-used directory paths
 *                          in the "Go to Directory" display for the
 *                          host associated with "dirwin".
 */
update_goto_dir_dialog(dirwin)
struct dirwin_st *dirwin;
{
	struct sl_struct *list;
	XmStringTable items;
	int i;
	Widget widget;

	/* Enter directory paths into scrolled list */
	retrieve_cache(DIRECTORY, hinfo[dirwin->host].hostname, &list, sort_caches);
	items = (XmStringTable)XtMalloc(sizeof(XmString)*list->nentries);
	for (i=0; i<list->nentries; i++)
		items[i] = XmStringCreateSimple(list->entries[i]);
	XtVaSetValues(
		w_gotoDirDialog,
		XmNlistItems,		items,
		XmNlistItemCount,	list->nentries,
		NULL
	);

	/* Free stuff */
	for (i=0; i<list->nentries; i++)
		XmStringFree(items[i]);
	XtFree((char *)items);
	release_array_list(list);

    /* Clear text */
    widget = XmSelectionBoxGetChild(w_gotoDirDialog, XmDIALOG_TEXT);
    if (hinfo[dirwin->host].system == SYS_VMS) {
		XmTextSetString(widget, "[.]");
		XmTextSetInsertionPosition(widget, 2);
	} else 
        XmTextSetString(widget, "");

	/* Clear selection */
	widget = XmSelectionBoxGetChild(w_gotoDirDialog, XmDIALOG_LIST);
	XtVaSetValues(widget, XmNselectedItemCount, 0, NULL);
}


/*
 * cb_goto_dir_ok - Callback for "Go to Directory" dialog's OK button.
 *                  This routine actually displays the directory.
 */
void
cb_goto_dir_ok(widget, client_data, call_data)
Widget widget;
XtPointer client_data;
XtPointer call_data;
{
	struct dirwin_st *dirwin = (struct dirwin_st *)client_data;
	char *path;
	char *full_path;
	int retval;

	/* Get directory path*/
	path = XmTextGetString(XmSelectionBoxGetChild(w_gotoDirDialog,
		XmDIALOG_TEXT));

	/* Get rid of dialog */
	XtDestroyWidget(XtParent(w_gotoDirDialog));

	/* Convert directory path to full path */
	retval = convert_to_full_path(dirwin, path, &full_path);
	XtFree(path);
	switch (retval) {
	case -1:
		record_and_alert("Unable to go to directory.", dirwin->w_shell);
		return;
	case -3:
		lost_connection(dirwin->host, dirwin);
		return;
	}

    /* Try to cd to specified directory */
	retval = display_dir(dirwin->host, dirwin, full_path, auto_refresh, True);
	XtFree(full_path);
	switch (retval) {
	case -1:
		record_and_alert("Unable to go to directory.", dirwin->w_shell);
		break;
	case -3:
		lost_connection(dirwin->host, dirwin);
	}
}


/*
 * cb_goto_dir_cancel - Callback to handle "Go to Directory" Dialog
 *                      "Cancel" button.
 */
void
cb_goto_dir_cancel(widget, client_data, call_data)
Widget widget;
XtPointer client_data;
XtPointer call_data;
{
	/* Get rid of "Go to Directory" Directory Dialog */
	XtDestroyWidget(XtParent(w_gotoDirDialog));
}


/*
 * cb_goto_dir_help - Callback that invokes the help package for the "Go to
 *                    Directory" dialog.
 */
void
cb_goto_dir_help(widget, client_data, call_data)
Widget widget;
XtPointer client_data;
XtPointer call_data;
{
    help_dialog("Go to Directory", help_goto_dir);
}


/*
 * cb_goto_prev_dir - Callback to go to the directory selected in the
 *                    "Go to Previous Directory" menu.
 */
void
cb_goto_prev_dir(widget, client_data, call_data)
Widget widget;
XtPointer client_data;
XtPointer call_data;
{
	XmString string;
	char *new_wd;
	struct dirwin_st *dirwin = (struct dirwin_st *)client_data;
	int retval;

	/* Clear error flag */
	raise_okflag();

	XtVaGetValues(widget, XmNlabelString, &string, NULL);
	new_wd = cstring_to_text(string);
	XmStringFree(string);   /* Yes, this is necessary */

    /* Try to cd to specified directory */
	retval = display_dir(dirwin->host, dirwin, new_wd, auto_refresh, True);
	XtFree(new_wd);
	switch (retval) {
	case -1:
		record_and_alert("Unable to go to directory.", dirwin->w_shell);
		break;
	case -3:
		lost_connection(dirwin->host, dirwin);
	}
}


/*
 * initialize_goto_menu - Initialize the "Go to Previous Directory" menu
 *                        by creating a bunch of placeholder menu items.
 */
initialize_goto_menu(dirwin)
struct dirwin_st *dirwin;
{
	XmString string;
	int i;
	Widget w_pulldown;

	/* Get pulldown menu */
	XtVaGetValues(dirwin->w_gotoMenu, XmNsubMenuId, &w_pulldown, NULL);

	/* Now create the menu items, but leave them unmanaged */
	string = XmStringCreateSimple("Dummy");
	for (i=0; i<MAXDIRCACHE; i++) {
		dirwin->w_gotoMenuItem[i] = XtVaCreateWidget(
            "",
            xmPushButtonGadgetClass,
            w_pulldown,
            XmNlabelString, string,
            NULL
        );
        XtAddCallback(dirwin->w_gotoMenuItem[i], XmNactivateCallback,
            cb_goto_prev_dir, (XtPointer)dirwin);
    }
    XmStringFree(string);
}


/*
 * update_goto_menus_for_host - Update the "Go To" menu in each of the 
 *                              specified host's directory windows.
 */
update_goto_menus_for_host(host)
int host;
{
	struct dirwin_st *dirwin;

	dirwin = dirwin_head;
	while (dirwin) {
		if (dirwin->host == host)
			update_goto_menu(dirwin);
		dirwin = dirwin->next;
	}
}


/*
 * update_all_goto_menus - Update the "Go To" menu in each directory window.
 */
update_all_goto_menus()
{
	struct dirwin_st *dirwin;

	dirwin = dirwin_head;
	while (dirwin) {
		update_goto_menu(dirwin);
		dirwin = dirwin->next;
	}
}


/*
 * update_goto_menu - Update the "Go to Previous Directory" menu with the
 *                    latest values from the directory cache.
 */
update_goto_menu(dirwin)
struct dirwin_st *dirwin;
{
	int i;
	struct sl_struct *list;
	XmString label;

    /* Get rid of current "Dir" menu items */
    for (i=0; i<MAXDIRCACHE; i++)
        XtUnmanageChild(dirwin->w_gotoMenuItem[i]);

    /* Retrieve cached directory paths */
    retrieve_cache(DIRECTORY, hinfo[dirwin->host].hostname, &list, sort_caches);

    /* Enter directory paths into "Ops" menu */
    for (i=0; i<list->nentries; i++) {
        label = XmStringCreateSimple(list->entries[i]);
        XtVaSetValues(dirwin->w_gotoMenuItem[i], XmNlabelString, label,
			NULL);
        XmStringFree(label);
        XtManageChild(dirwin->w_gotoMenuItem[i]);
    }

	/* Free stuff */
	release_array_list(list);
}


/*
 * convert_to_full_path - Convert "path" to the equivalent full path
 *                        relative the dirwin's directory path.   Returns
 *                        0 for success, -3 for lost connection, and -1
 *                        for other errors.  If successful, full path is
 *                        returned in "full_path", which can be released
 *                        with XtFree().
 */
convert_to_full_path(dirwin, path, full_path)
struct dirwin_st *dirwin;
char *path;
char **full_path;
{
	int host = dirwin->host;
	int retval;

    /* This might take some time */
    use_busy_cursor();

    /* If VMS or if partial path, first cd into working directory */
    if ((hinfo[host].system == SYS_VMS)||(strlen(path) && path[0] != '/')) {
        if (host == LOCAL) {
            if ((retval = local_cd(dirwin->dirname, True)) < 0)
				goto done;
        } else if ((retval = remote_cd(host, dirwin->dirname, False, True)) < 0)
			goto done;
    }

    /* cd into new directory */
    if (host == LOCAL) {
        if ((retval = local_cd(path, True)) < 0)
			goto done;
        retval = local_pwd(full_path);
    } else {
		if ((retval = remote_cd(host, path, False, True)) < 0)
			goto done;
        retval = remote_pwd(host, full_path);
    }

done:

	restore_prev_cursor();
	return retval;
}


/*
 * cb_goto_parent - Callback to display directory's parent.
 */
void
cb_goto_parent(widget, client_data, call_data)
Widget widget;
XtPointer client_data;
XtPointer call_data;
{
    struct dirwin_st *dirwin = (struct dirwin_st *)client_data;
    char **wd_links;
    int nlinks = 0;
    int i;
    int len = 0;
    char *path;

    /* Clear error flag */
    raise_okflag();

    /* Parse current directory path. */
    wd_links = path_to_links(hinfo[dirwin->host].system,dirwin->dirname);

    /* Determine number of links in parent's path name */
    while (wd_links[nlinks])
        nlinks++;
    nlinks--;

    /* If no links, no parent to display */
    if (nlinks == 0) {
		beep();
		warn("No parent to display", dirwin->w_shell);
        return;
    }

    /* Determine length of path name */
    for (i=0; i<nlinks; i++)
        len += strlen(wd_links[i])+1;

    /* Build path name for parent directory */
	path = links_to_path(hinfo[dirwin->host].system, wd_links, nlinks);

    /* Free up memory returned by path_to_links */
    release_path_links(wd_links);

    /* Try to cd to parent directory */
	switch (display_dir(dirwin->host, dirwin, path, auto_refresh, True)) {
	case -1:
		record_and_alert("Unable to go to directory.", dirwin->w_shell);
		break;
	case -3:
		lost_connection(dirwin->host, dirwin);
	}
	XtFree(path);
}


/*
 * cb_goto_home - Callback to display home directory.
 */
void
cb_goto_home(widget, client_data, call_data)
Widget widget;
XtPointer client_data;
XtPointer call_data;
{
    struct dirwin_st *dirwin = (struct dirwin_st *)client_data;
	int retval;

    /* Clear error flag */
    raise_okflag();

    /* Try to cd to specified directory */
	retval = display_dir(dirwin->host, dirwin, hinfo[dirwin->host].homedir,
		auto_refresh, True);
	switch (retval) {
	case -1:
		record_and_alert("Unable to go to directory.", dirwin->w_shell);
		break;
	case -3:
		lost_connection(dirwin->host, dirwin);
	}
}

