/***************************************************************************/
/***************************************************************************/
/*                                                                         */
/*   (c) 1995.  The Regents of the University of California.  All rights   */
/*   reserved.                                                             */
/*                                                                         */
/*   This work was produced at the University of California, Lawrence      */
/*   Livermore National Laboratory (UC LLNL) under contract no.            */
/*   W-7405-ENG-48 (Contract 48) between the U.S. Department of Energy     */
/*   (DOE) and The Regents of the University of California (University)    */
/*   for the operation of UC LLNL.  Copyright is reserved to the           */
/*   University for purposes of controlled dissemination,                  */
/*   commercialization through formal licensing, or other disposition      */
/*   under terms of Contract 48; DOE policies, regulations and orders;     */
/*   and U.S. statutes.  The rights of the Federal Government are          */
/*   reserved under Contract 48 subject to the restrictions agreed upon    */
/*   by the DOE and University.                                            */
/*                                                                         */
/*                                                                         */
/*                              DISCLAIMER                                 */
/*                                                                         */
/*   This software was prepared as an account of work sponsored by an      */
/*   agency of the United States Government.  Neither the United States    */
/*   Government nor the University of California nor any of their          */
/*   employees, makes any warranty, express or implied, or assumes any     */
/*   liability or responsibility for the accuracy, completeness, or        */
/*   usefulness of any information, apparatus, product, or process         */
/*   disclosed, or represents that its specific commercial products,       */
/*   process, or service by trade name, trademark, manufacturer, or        */
/*   otherwise, does not necessarily constitute or imply its               */
/*   endorsement, recommendation, or favoring by the United States         */
/*   Government or the University of California. The views and opinions    */
/*   of the authors expressed herein do not necessarily state or reflect   */
/*   those of the United States Government or the University of            */
/*   California, and shall not be used for advertising or product          */
/*   endorsement purposes.                                                 */
/*                                                                         */
/*   Permission to use, copy, modify and distribute this software and its  */
/*   documentation for any non-commercial purpose, without fee, is         */
/*   hereby granted, provided that the above copyright notice and this     */
/*   permission notice appear in all copies of the software and            */
/*   supporting documentation, and that all UC LLNL identification in      */
/*   the user interface remain unchanged.  The title to copyright LLNL     */
/*   XDIR shall at all times remain with The Regents of the University     */
/*   of California and users agree to preserve same. Users seeking the     */
/*   right to make derivative works with LLNL XDIR for commercial          */
/*   purposes may obtain a license from the Lawrence Livermore National    */
/*   Laboratory's Technology Transfer Office, P.O. Box 808, L-795,         */
/*   Livermore, CA 94550.                                                  */
/*                                                                         */
/***************************************************************************/
/***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <errno.h>
#include <Xm/Xm.h>
#include "xdir.h"
#include "xfer.h"
#include "str.h"
#include "list.h"

#ifndef P_tmpdir
#define P_tmpdir  "/usr/tmp/"
#endif

#define MAXTRIES  10

int init_local_copy();
int do_local_copy();
int abort_local_copy();

int init_get();
int do_get();
int abort_get();

static int (*init_fn[2])() =  { init_local_copy,	init_get  };

static int (*do_fn[2])() =    { do_local_copy,		do_get    };

static int (*abort_fn[2])() = { abort_local_copy,	abort_get };

char **path_to_links();
char *merge_paths();
int cb_xfer_files();
char *unix_rel_dir_path_to_native();
char *unix_rel_file_path_to_native();
char *unix_links_to_path();
double stop_timer();
char *temp_file_name();
int cb_view_files();

extern int xfer_mode;
extern struct st_host_info hinfo[];
extern struct xfer_ctrl_block xfer_ctrl;
extern XtAppContext app;
extern Display *display;
extern int beep_when_ops_done;
extern int diagnostics;
extern int max_ftp_retries;
extern int preserve_vms_ver_nums;


/*
 * init_view - Initialize file transfer of selected files in directory
 *             window "dirwin" to temporary directory on local host
 *             for viewing.  "list" is an array list containing the
 *             relative paths of the files to view.  Returns 0 if
 *             successful, -3 for lost connection, and -1 for other
 *             errors.
 */
init_view(dirwin, list)
struct dirwin_st *dirwin;
struct sl_struct *list;
{
	int host = dirwin->host;
	int retval;
	struct entry_link *head;
	int i;

    /* This might take some time */
    use_busy_cursor();

	/* Set transfer mode */
	if ((host != LOCAL) && ((retval = ftp_type(host, xfer_mode)) < 0)) {
		restore_prev_cursor();
		return retval;
	}

    /* Pop up transfer monitor */
    show_xfer_monitor(dirwin);
    clear_xfer_monitor();
	XSync(display, 0);
	sleep(1);

    /* Disable controls */
    enable_controls(False);

    /* Form list of selected items to transfer */
	head = NULL;
	for (i=list->nentries-1; i>=0; i--)
		add_to_linked_list(&head, list->entries[i]);

	/* Set up transfer control block */
	xfer_ctrl.operation = COPY;
	xfer_ctrl.mode = xfer_mode;
	xfer_ctrl.level = 0;
	xfer_ctrl.state = 0;
	xfer_ctrl.src_dirwin = dirwin;
	xfer_ctrl.head[0] = head;
	xfer_ctrl.abort_requested = False;
	xfer_ctrl.src_host_system = hinfo[dirwin->host].system;
	xfer_ctrl.src_host_server = hinfo[dirwin->host].server;
	xfer_ctrl.snk_host_system = SYS_UNIX;
	xfer_ctrl.src_dirname = XtNewString(dirwin->dirname);
	if (dirwin->host == LOCAL)
		xfer_ctrl.src_host_type = XLOCAL;
	else
		xfer_ctrl.src_host_type = XREMOTE;

	/* Transfer file */
	XtAppAddWorkProc(app, (XtWorkProc)cb_view_files, NULL);
	return 0;
}


/*
 * cb_view_files - Work proc for transferring and viewing files.
 */
cb_view_files()
{
	double ftime;
	struct entry_link *ptr;
	int src_host = xfer_ctrl.src_dirwin->host;
	int src_host_system = xfer_ctrl.src_host_system;
	int snk_host_system = xfer_ctrl.snk_host_system;
	struct dirwin_st *src_dirwin =  xfer_ctrl.src_dirwin;
	char *src_dirname = xfer_ctrl.src_dirname;
	int src_host_type = xfer_ctrl.src_host_type;
	char msg[MAXPATHLEN+40];
	char **rel_path_links;
	char *q;
	char *temp_dir;
	char *temp_path;
	char **link_ptr;
	int nlinks;
	char *dir;
	int retval1;
	int retval2;

	switch (xfer_ctrl.state) {
	case 0:   /* State 0: No transfer in progress */
		xfer_ctrl.unix_src_rel_path = NULL;
		xfer_ctrl.unix_snk_rel_path = NULL;
		xfer_ctrl.src_path = NULL;
		xfer_ctrl.snk_path = NULL;
		xfer_ctrl.src_rel_path = NULL;
		xfer_ctrl.snk_rel_path = NULL;
		xfer_ctrl.snk_dirname = NULL;
		if (xfer_ctrl.head[0] == NULL)
			goto done;
		if (xfer_ctrl.abort_requested)
			goto done;

		/* Create temporary directory name */
		if ((xfer_ctrl.snk_dirname = temp_file_name()) == NULL) {
			sprintf(msg, "Can't create temp directory for %s.",
				xfer_ctrl.head[0]->entry);
			record_warning(msg);
			sprintf(msg, "Unable to view file %s.", xfer_ctrl.head[0]->entry);
			record_and_alert(msg, src_dirwin->w_shell);
			ptr = xfer_ctrl.head[0];
			xfer_ctrl.head[0] = ptr->next;
			XtFree(ptr->entry);
			XtFree((char *)ptr);
			return False;
		}

		/* Build relative source path name (in Unix format) */
		rel_path_links = path_to_links(src_host_system,
			xfer_ctrl.head[0]->entry);
		nlinks = 0;
		link_ptr = rel_path_links;
		while (*link_ptr) {
			nlinks++;
			if (src_host_system == SYS_VMS) {  /* Shave off ".dir;n" */
				if ((q = strstr(*link_ptr, ".dir")))
					*q = '\0';
			}
			link_ptr++;
		}
	    xfer_ctrl.unix_src_rel_path = unix_links_to_path(rel_path_links,nlinks);
		release_path_links(rel_path_links);

		/* Build relative sink path name (in Unix format) */
	    rel_path_links = (char **)XtMalloc(sizeof(char *)*2);
		temp_path = merge_paths(snk_host_system, xfer_ctrl.snk_dirname,
			xfer_ctrl.head[0]->entry);
		parse_path(snk_host_system, temp_path, &temp_dir, &rel_path_links[0]);
		XtFree(temp_path);
		XtFree(temp_dir);
		if (src_host_system == SYS_VMS) {  /* Shave off ".dir;n" */
			if ((q = strstr(rel_path_links[0], ".dir")))
				*q = '\0';
		}
		rel_path_links[1] = NULL;
	    xfer_ctrl.unix_snk_rel_path = unix_links_to_path(rel_path_links, 1);
		release_path_links(rel_path_links);

		/* Build relative and full source paths (and assume it is a file) */
		xfer_ctrl.src_rel_path = unix_rel_file_path_to_native(
			src_host_system, xfer_ctrl.unix_src_rel_path);
    	xfer_ctrl.src_path = merge_paths(src_host_system,
			src_dirname, xfer_ctrl.src_rel_path);

        /* Build relative and full sink paths (and assume it is a file) */
		xfer_ctrl.snk_rel_path = unix_rel_file_path_to_native(
			snk_host_system, xfer_ctrl.unix_snk_rel_path);
   		xfer_ctrl.snk_path = merge_paths(snk_host_system,
			xfer_ctrl.snk_dirname, xfer_ctrl.snk_rel_path);
		if (src_host_system == SYS_VMS) {
			if ((q = strrchr(xfer_ctrl.snk_path, ';')))
				*q = '\0';
		}

		/* Initialize file transfer */
		init_xfer_monitor1();
		parse_path(SYS_UNIX, xfer_ctrl.snk_path, &dir, NULL);
		retval1 = local_mkdir(dir, 0700);
		XtFree(dir);
		if (retval1 || ((retval2 = (*init_fn[src_host_type])()) == -1)) {
			sprintf(msg, "Unable to initialize transfer of %s.",
						 xfer_ctrl.head[0]->entry);
			record_warning(msg);
			sprintf(msg, "Unable to view file %s.", xfer_ctrl.head[0]->entry);
			record_and_alert(msg, src_dirwin->w_shell);
			delete_file_and_its_dir(xfer_ctrl.snk_path);
			xfer_ctrl.state = 0;
			ptr = xfer_ctrl.head[0];
			xfer_ctrl.head[0] = ptr->next;
			XtFree(xfer_ctrl.unix_src_rel_path);
			XtFree(xfer_ctrl.unix_snk_rel_path);
			XtFree(xfer_ctrl.src_path);
			XtFree(xfer_ctrl.snk_path);
			XtFree(xfer_ctrl.src_rel_path);
			XtFree(xfer_ctrl.snk_rel_path);
			XtFree(xfer_ctrl.snk_dirname);
			XtFree(ptr->entry);
			XtFree((char *)ptr);
			return False;
		} else if (retval2 == -3)
			goto lost;
		init_xfer_monitor2();
		start_timer();
		xfer_ctrl.state = 1;

	case 1:   /* State 1: transfer in progress */
		if (xfer_ctrl.abort_requested)
			goto abort;
		retval1 = (*do_fn[src_host_type])();
		switch (retval1) {
		case -3:
			goto lost;
		case -1:
			sprintf(msg, "Unable to complete transfer of %s.",
						 xfer_ctrl.head[0]->entry);
			record_warning(msg);
			sprintf(msg, "Unable to view file %s.", xfer_ctrl.head[0]->entry);
			record_and_alert(msg, src_dirwin->w_shell);
			delete_file_and_its_dir(xfer_ctrl.snk_path);
			ptr = xfer_ctrl.head[0];
			xfer_ctrl.head[0] = ptr->next;
			XtFree(xfer_ctrl.unix_src_rel_path);
			XtFree(xfer_ctrl.unix_snk_rel_path);
			XtFree(xfer_ctrl.src_path);
			XtFree(xfer_ctrl.snk_path);
			XtFree(xfer_ctrl.src_rel_path);
			XtFree(xfer_ctrl.snk_rel_path);
			XtFree(xfer_ctrl.snk_dirname);
			XtFree(ptr->entry);
			XtFree((char *)ptr);
			xfer_ctrl.state = 0;
			return False;
		case 0:   /* Transfer complete */
			update_xfer_monitor();
			ftime = stop_timer();
			if (diagnostics >= NORMAL) {
				sprintf(msg, "*** Successfully transferred file:  %s\n",
						xfer_ctrl.src_rel_path);
				write_log(msg);
				print_xfer_stats(ftime, xfer_ctrl.file_len);
			}
			view_file(src_dirwin, xfer_ctrl.snk_path);
			ptr = xfer_ctrl.head[0];
			xfer_ctrl.head[0] = ptr->next;
			XtFree(xfer_ctrl.unix_src_rel_path);
			XtFree(xfer_ctrl.unix_snk_rel_path);
			XtFree(xfer_ctrl.src_path);
			XtFree(xfer_ctrl.snk_path);
			XtFree(xfer_ctrl.src_rel_path);
			XtFree(xfer_ctrl.snk_rel_path);
			XtFree(xfer_ctrl.snk_dirname);
			XtFree(ptr->entry);
			XtFree((char *)ptr);
			xfer_ctrl.state = 0;
			return False;
		case 1:   /* More to transfer */
			update_xfer_monitor();
			return False;
		}
	}

done:
	
	if (xfer_ctrl.abort_requested) {
		write_log("*** Viewing of file(s) aborted.");
		hide_abort_dialog();
	} else
		hide_xfer_monitor();
	enable_controls(True);
	clear_selected_entries();
	goto finish;

abort:

	write_log("*** Viewing of file(s) aborted.");
	if ((*abort_fn[src_host_type])() == -3) 
		goto lost;
	else {
		delete_file_and_its_dir(xfer_ctrl.snk_path);
		hide_abort_dialog();
		enable_controls(True);
		clear_selected_entries();
		goto finish;
	}

lost:

	delete_file_and_its_dir(xfer_ctrl.snk_path);
	if (xfer_ctrl.abort_requested)
		hide_abort_dialog();
	else
		hide_xfer_monitor();
	enable_controls(True);
	clear_selected_entries();
	restore_prev_cursor();
	lost_connection(src_host, src_dirwin);
	use_busy_cursor();

finish:

	/* Finish up */
	clean_up_xfer_ctrl();
	update_dir_controls();
	if (beep_when_ops_done)
		beep();
	restore_prev_cursor();
	return True;
}


/*
 * temp_file_name - Create a unique name for a temporary file in the
 *                  directory defined as P_tmpdir in <stdio.h>.  If the
 *                  environment variable TMPDIR is defined, it is used
 *                  as the directory.  If successful, this function returns
 *                  a pointer to the name, which can be released by calling
 *                  XtFree(); if unsuccessful, NULL is returned.  The name
 *                  will begin with the prefix "xdir".
 */
char *
temp_file_name()
{
	char *dir_path = NULL;
	char *file_path;
	char *temp;
	int pid;
	char pid_string[5];
	char suffix[5];
	int i;
	int j;

	/* Get directory path */
	if ((temp = getenv("TMPDIR")))
		dir_path = XtNewString(temp);
	else
		dir_path = XtNewString(P_tmpdir);

	/* Verify that directory is accessible */
	if (access(dir_path, 0) < 0) {
		XtFree(dir_path);
		return NULL;
	}

	/* Convert PID into string */
	pid = getpid();
	sprintf(pid_string, "%04d", pid%10000);

	/* Try several times to create a unique name */
	for (i=0; i<MAXTRIES; i++) {
		for (j=0; j<4; j++)
			suffix[j] = ((rand()&0xff)%26)+'A';
		suffix[4] = '\0';
		file_path = XtMalloc(strlen(dir_path)+14);
		strcpy(file_path, dir_path);
		if (file_path[strlen(file_path)-1] != '/')
			strcat(file_path, "/");
		strcat(file_path, "xdir");
		strcat(file_path, pid_string);
		strcat(file_path, suffix);
		if (access(file_path, 0) < 0) {
			if (errno == ENOENT) {
				XtFree(dir_path);
				return file_path;
			}
		}
		XtFree(file_path);
	}

	XtFree(dir_path);
	return NULL;
}

