/***************************************************************************/
/***************************************************************************/
/*                                                                         */
/*   (c) 1995.  The Regents of the University of California.  All rights   */
/*   reserved.                                                             */
/*                                                                         */
/*   This work was produced at the University of California, Lawrence      */
/*   Livermore National Laboratory (UC LLNL) under contract no.            */
/*   W-7405-ENG-48 (Contract 48) between the U.S. Department of Energy     */
/*   (DOE) and The Regents of the University of California (University)    */
/*   for the operation of UC LLNL.  Copyright is reserved to the           */
/*   University for purposes of controlled dissemination,                  */
/*   commercialization through formal licensing, or other disposition      */
/*   under terms of Contract 48; DOE policies, regulations and orders;     */
/*   and U.S. statutes.  The rights of the Federal Government are          */
/*   reserved under Contract 48 subject to the restrictions agreed upon    */
/*   by the DOE and University.                                            */
/*                                                                         */
/*                                                                         */
/*                              DISCLAIMER                                 */
/*                                                                         */
/*   This software was prepared as an account of work sponsored by an      */
/*   agency of the United States Government.  Neither the United States    */
/*   Government nor the University of California nor any of their          */
/*   employees, makes any warranty, express or implied, or assumes any     */
/*   liability or responsibility for the accuracy, completeness, or        */
/*   usefulness of any information, apparatus, product, or process         */
/*   disclosed, or represents that its specific commercial products,       */
/*   process, or service by trade name, trademark, manufacturer, or        */
/*   otherwise, does not necessarily constitute or imply its               */
/*   endorsement, recommendation, or favoring by the United States         */
/*   Government or the University of California. The views and opinions    */
/*   of the authors expressed herein do not necessarily state or reflect   */
/*   those of the United States Government or the University of            */
/*   California, and shall not be used for advertising or product          */
/*   endorsement purposes.                                                 */
/*                                                                         */
/*   Permission to use, copy, modify and distribute this software and its  */
/*   documentation for any non-commercial purpose, without fee, is         */
/*   hereby granted, provided that the above copyright notice and this     */
/*   permission notice appear in all copies of the software and            */
/*   supporting documentation, and that all UC LLNL identification in      */
/*   the user interface remain unchanged.  The title to copyright LLNL     */
/*   XDIR shall at all times remain with The Regents of the University     */
/*   of California and users agree to preserve same. Users seeking the     */
/*   right to make derivative works with LLNL XDIR for commercial          */
/*   purposes may obtain a license from the Lawrence Livermore National    */
/*   Laboratory's Technology Transfer Office, P.O. Box 808, L-795,         */
/*   Livermore, CA 94550.                                                  */
/*                                                                         */
/***************************************************************************/
/***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <errno.h>
#include <Xm/Xm.h>
#include "xdir.h"
#include "xfer.h"
#include "str.h"

int init_local_copy();
int do_local_copy();
int abort_local_copy();

int init_put();
int do_put();
int abort_put();

int init_get();
int do_get();
int abort_get();

int init_remote_copy();
int do_remote_copy();
int abort_remote_copy();

static int (*init_fn[2][2])() = {
	{ init_local_copy,  init_put         },
	{ init_get,         init_remote_copy }
};

static int (*do_fn[2][2])() = {
	{ do_local_copy,    do_put         },
	{ do_get,           do_remote_copy }
};

static int (*abort_fn[2][2])() = {
	{ abort_local_copy, abort_put         },
	{ abort_get,        abort_remote_copy }
};

static char *not_implemented_msg = "Copying entry(s) from a remote host to itself not implemented.\nConsider moving file(s) instead.";

static char *xfer_question = "You are attempting to copy a directory, but\nthe general preference COPY MODE is set to\nNONRECURSIVE.\n\nCopy directories anyway?";

static char *msg1 = "Unable to redisplay directory.\n\nDirectory window might be out of date.";

char *merge_paths();
int cb_xfer_files();
char *entry_to_rel_path();
char *unix_rel_dir_path_to_native();
char *unix_rel_file_path_to_native();
char *unix_links_to_path();
double stop_timer();
struct ref_st *create_reference_subtree();
char **path_to_links();
char *entry_to_full_path();

struct xfer_ctrl_block xfer_ctrl;

static struct timeval start_time;
static struct timeval stop_time;

extern int rcopy_mode;
extern int inquire_on_copy;
extern int xfer_mode;
extern struct st_host_info hinfo[];
extern struct xfer_ctrl_block xfer_ctrl;
extern XtAppContext app;
extern Display *display;
extern int beep_when_ops_done;
extern int diagnostics;
extern int max_ftp_retries;
extern int preserve_vms_ver_nums;


/*
 * init_copy - Initialize file transfer of selected files in directory
 *             window "src_dirwin" to directory window "snk_dirwin".  If
 *             "einfo" is non-NULL then transfer the files into the
 *             directory represented by "einfo".
 */
init_copy(src_dirwin, snk_dirwin, einfo)
struct dirwin_st *src_dirwin;
struct dirwin_st *snk_dirwin;
struct entry_info *einfo;
{
	int src_host = src_dirwin->host;
	int snk_host = snk_dirwin->host;
	int retval;
	char *temp_snk_dir;
	char *snk_dir;
	struct entry_link *head;
	struct entry_link *tail;
	struct entry_link *ptr;
	int i;
	char *q;
	char **links;
	char **link_ptr;
	int nlinks;

	/* Is operation implemented */
	if ((src_host != LOCAL) && (src_host == snk_host)) {
		warn(not_implemented_msg, snk_dirwin->w_shell);
		return;
	}

	/* Ask the user if okay to copy */
	if (inquire_on_copy) {
		if (!verify_selection(src_dirwin, snk_dirwin,
				"Do you really want to copy these entries?"))
			return;
	}

    /* This might take some time */
    use_busy_cursor();

	/* Set transfer mode */
	if ((src_host != LOCAL) && (ftp_type(src_host, xfer_mode) < 0)) {
		restore_prev_cursor();
		lost_connection(src_host, snk_dirwin);
		return;
	}
	if ((snk_host != LOCAL) && (ftp_type(snk_host, xfer_mode) < 0)) {
		restore_prev_cursor();
		lost_connection(snk_host, snk_dirwin);
		return;
	}

	/* Determine true sink directory path */
	if (einfo)
		temp_snk_dir = entry_to_full_path(einfo);
	else 
		temp_snk_dir = XtNewString(snk_dirwin->dirname);
    retval = cd(snk_dirwin->host, temp_snk_dir, True, &snk_dir);
	XtFree(temp_snk_dir);
    if (retval == -1) {
		restore_prev_cursor();
        record_and_alert("Unable to transfer file(s).", snk_dirwin->w_shell);
        return;
    } else if (retval == -3) {
		restore_prev_cursor();
		lost_connection(snk_dirwin->host, snk_dirwin);
		return;
	}

    /* Pop up transfer monitor */
    show_xfer_monitor(snk_dirwin);
    clear_xfer_monitor();
	XSync(display, 0);
	sleep(1);

    /* Disable controls */
    enable_controls(False);

    /* Form list of selected items to transfer in Unix format */
    head = NULL;
    tail = NULL;
	for (i=0; i<src_dirwin->nentries; i++)
		if (src_dirwin->entries[i].level == 0)
		    postorder_selected_items(&src_dirwin->entries[i], &head, &tail);
	ptr = head;
	while (ptr) {
		links = path_to_links(hinfo[src_dirwin->host].system, ptr->entry);
		XtFree(ptr->entry);
		nlinks = 0;
		link_ptr = links;
		while (*link_ptr++)
			nlinks++;
		if (hinfo[src_dirwin->host].system == SYS_VMS)   /* Remove ".dir;n" */
			for (i=0; i<nlinks; i++)
				if ((q = strstr(links[i], ".dir")))
					*q = '\0';
		ptr->entry = unix_links_to_path(links, nlinks);
		release_path_links(links);
		ptr = ptr->next;
	}

	/* Set up transfer control block */
	if (rcopy_mode)
		xfer_ctrl.operation = RCOPY;
	else
		xfer_ctrl.operation = COPY;
	xfer_ctrl.mode = xfer_mode;
	xfer_ctrl.level = 0;
	xfer_ctrl.state = 0;
	xfer_ctrl.nretries = 0;
	xfer_ctrl.error = False;
	xfer_ctrl.asked_user = False;
	xfer_ctrl.src_dirwin = src_dirwin;
	xfer_ctrl.snk_dirwin = snk_dirwin;
	xfer_ctrl.head[0] = head;
	xfer_ctrl.abort_requested = False;
	xfer_ctrl.src_host_system = hinfo[src_dirwin->host].system;
	xfer_ctrl.src_host_server = hinfo[src_dirwin->host].server;
	xfer_ctrl.snk_host_system = hinfo[snk_dirwin->host].system;
	xfer_ctrl.src_dirname = XtNewString(src_dirwin->dirname);
	xfer_ctrl.snk_dirname = snk_dir;
	if (src_dirwin->host == LOCAL)
		xfer_ctrl.src_host_type = XLOCAL;
	else
		xfer_ctrl.src_host_type = XREMOTE;
	if (snk_dirwin->host == LOCAL)
		xfer_ctrl.snk_host_type = XLOCAL;
	else
		xfer_ctrl.snk_host_type = XREMOTE;

	/* Transfer file */
	XtAppAddWorkProc(app, (XtWorkProc)cb_xfer_files, NULL);
}


/*
 * cb_xfer_files - Work proc for transferring files.
 */
cb_xfer_files()
{
	double ftime;
	struct entry_link *ptr;
	int src_host = xfer_ctrl.src_dirwin->host;
	int snk_host = xfer_ctrl.snk_dirwin->host;
	int src_host_system = xfer_ctrl.src_host_system;
	int snk_host_system = xfer_ctrl.snk_host_system;
	struct dirwin_st *src_dirwin =  xfer_ctrl.src_dirwin;
	struct dirwin_st *snk_dirwin =  xfer_ctrl.snk_dirwin;
	char *src_dirname = xfer_ctrl.src_dirname;
	char *snk_dirname = xfer_ctrl.snk_dirname;
	int src_host_type = xfer_ctrl.src_host_type;
	int snk_host_type = xfer_ctrl.snk_host_type;
	struct sl_struct *list;
	int retval;
	int lost_src_host = False;
	int lost_snk_host = False;
	int i;
	int is_a_directory;
	char msg[MAXPATHLEN+50];
	char **rel_path_links;
	char **first_path_links;
	char *q;
	struct dirwin_st *dirwin;
	struct entry_info *einfo;
	int indx;
	char *anchor_rel_path;
	struct ref_st *ref_tree;
	char *temp_dir;
	char *temp_path;
	int nlinks;
	char **link_ptr;
	char **p;

	switch (xfer_ctrl.state) {
	case 0:   /* State 0: No transfer in progress */
		xfer_ctrl.unix_src_rel_path = NULL;
		xfer_ctrl.unix_snk_rel_path = NULL;
		xfer_ctrl.src_path = NULL;
		xfer_ctrl.snk_path = NULL;
		xfer_ctrl.src_rel_path = NULL;
		xfer_ctrl.snk_rel_path = NULL;
		if (xfer_ctrl.head[xfer_ctrl.level] == NULL) {
			if (xfer_ctrl.level == 0) {
				if (xfer_ctrl.error)
					record_and_alert("At least one file transfer failed.",
						snk_dirwin->w_shell);
				goto done;
			} else {
				xfer_ctrl.level--;
				ptr = xfer_ctrl.head[xfer_ctrl.level];
				xfer_ctrl.head[xfer_ctrl.level] = ptr->next;
				XtFree(ptr->entry);
				XtFree((char *)ptr);
				return False;
			}
		}
		if (xfer_ctrl.abort_requested)
			goto done;

		/* Build relative source path name (in Unix format) */
		first_path_links = path_to_links(src_host_system,
			xfer_ctrl.head[0]->entry);
		nlinks = 0;
		link_ptr = first_path_links;
		while (*link_ptr++)
			nlinks++;
		nlinks += xfer_ctrl.level;
		rel_path_links = (char **)XtMalloc(sizeof(char *)*(nlinks+1));
		indx = 0;
		p = first_path_links;
		while (*p) {
			rel_path_links[indx++] = XtNewString(*p);
			p++;
		}
		release_path_links(first_path_links);
		for (i=1; i<=xfer_ctrl.level; i++)
			rel_path_links[indx++] = XtNewString(xfer_ctrl.head[i]->entry);
		if (src_host_system == SYS_VMS)   /* Shave off ".dir;n" */
			for (i=0; i<nlinks; i++) 
				if ((q = strstr(rel_path_links[i], ".dir")))
					*q = '\0';
		rel_path_links[nlinks] = NULL;
	    xfer_ctrl.unix_src_rel_path = unix_links_to_path(rel_path_links,nlinks);
		release_path_links(rel_path_links);

		/* Build relative sink path name (in Unix format) */
	    rel_path_links = (char **)XtMalloc(sizeof(char *)*(xfer_ctrl.level+2));
		temp_path = merge_paths(src_host_system, src_dirname,
			xfer_ctrl.head[0]->entry);
		parse_path(src_host_system, temp_path, &temp_dir, &rel_path_links[0]);
		XtFree(temp_path);
		XtFree(temp_dir);
	    for (i=1; i<=xfer_ctrl.level; i++)
	        rel_path_links[i] = XtNewString(xfer_ctrl.head[i]->entry);
	    for (i=0; i<=xfer_ctrl.level; i++) {
			if (src_host_system == SYS_VMS) {  /* Shave off ".dir;n" */
				if ((q = strstr(rel_path_links[i], ".dir")))
					*q = '\0';
			}
		}
		rel_path_links[xfer_ctrl.level+1] = NULL;
	    xfer_ctrl.unix_snk_rel_path = unix_links_to_path(rel_path_links,
			xfer_ctrl.level+1);
		release_path_links(rel_path_links);

		/* Build relative source path (and assume it is a directory) */
		xfer_ctrl.src_rel_path = unix_rel_dir_path_to_native(src_host_system,
			xfer_ctrl.unix_src_rel_path);
		/* Build source path (and assume it is a directory) */
    	xfer_ctrl.src_path = merge_paths(src_host_system, src_dirname,
			xfer_ctrl.src_rel_path);

		/* Test for directory by trying to cd to source path */
		is_a_directory = False;
		if (src_host == LOCAL) {
			if (local_cd(xfer_ctrl.src_path, False) == 0)
				is_a_directory = True;
		} else
			switch (remote_cd(src_host, xfer_ctrl.src_path, True, False)) {
			case -3:
				lost_src_host = True;
				goto lost;
			case 0:
				is_a_directory = True;
			}

		/* If it is a directory, transfer it if that is what user wants */
		if (is_a_directory) {
			if ((xfer_ctrl.operation != RCOPY) && !xfer_ctrl.asked_user) {
				beep();
				if (question_dialog(xfer_question, snk_dirwin->w_shell))
					xfer_ctrl.operation = RCOPY;
				xfer_ctrl.asked_user = True;
			}
			if (xfer_ctrl.operation == RCOPY) {
				/* Build relative sink path */
				xfer_ctrl.snk_rel_path = unix_rel_dir_path_to_native(
					snk_host_system, xfer_ctrl.unix_snk_rel_path);
				/* Build sink path */
		    	xfer_ctrl.snk_path = merge_paths(snk_host_system,
					snk_dirname, xfer_ctrl.snk_rel_path);
				/* Make sink directory */
				if (snk_host == LOCAL)
					retval = local_mkdir(xfer_ctrl.snk_path, 0777);
				else
					retval = remote_mkdir(snk_host, xfer_ctrl.snk_path);
				switch (retval) {
				case -3:
					lost_snk_host = True;
					goto lost;
				case -1:
					sprintf(msg,
						"File transfers aborted.  Can't make directory %s.",
						xfer_ctrl.snk_path);
					record_and_alert(msg, xfer_ctrl.snk_dirwin->w_shell,
						snk_dirwin->w_shell);
					goto done;
				default:
					if (diagnostics >= NORMAL) {
						sprintf(msg, "*** Successfully created directory: %s\n",
								xfer_ctrl.snk_rel_path);
						write_log(msg);
					}
				}
				/* Get source directory list */
				if (src_host == LOCAL)
					retval = local_ls(xfer_ctrl.src_path, &list, False, True);
				else
					retval = remote_ls(src_host, xfer_ctrl.src_path, &list,
						False, True);
				switch (retval) {
				case -3:
					lost_src_host = True;
					goto lost;
				case -1:
					record_and_alert(
						"File transfer(s) aborted.  Can't get directory list.",
						src_dirwin->w_shell);
					goto done;
				}
				/* Add source directory list to new level */
				xfer_ctrl.level++;
				if (xfer_ctrl.level == MAXLEVELS) {
					record_and_alert(
						"File transfer(s) aborted.  Recursion too deep.",
						src_dirwin->w_shell);
					goto done;
				}
				xfer_ctrl.head[xfer_ctrl.level] = NULL;
				for (i=list->nentries-1; i>=0; i--)
					add_to_linked_list(&xfer_ctrl.head[xfer_ctrl.level],
						list->entries[i]);
				release_array_list(list);
				XtFree(xfer_ctrl.unix_src_rel_path);
				XtFree(xfer_ctrl.unix_snk_rel_path);
				XtFree(xfer_ctrl.src_path);
				XtFree(xfer_ctrl.snk_path);
				XtFree(xfer_ctrl.src_rel_path);
				XtFree(xfer_ctrl.snk_rel_path);
				return False;
			}
		}
		XtFree(xfer_ctrl.src_path);
		XtFree(xfer_ctrl.src_rel_path);

		/* Build relative and full source paths (and assume it is a file) */
		xfer_ctrl.src_rel_path = unix_rel_file_path_to_native(
			src_host_system, xfer_ctrl.unix_src_rel_path);
    	xfer_ctrl.src_path = merge_paths(src_host_system,
			src_dirname, xfer_ctrl.src_rel_path);

        /* Build relative and full sink paths (and assume it is a file) */
		xfer_ctrl.snk_rel_path = unix_rel_file_path_to_native(
			snk_host_system, xfer_ctrl.unix_snk_rel_path);
   		xfer_ctrl.snk_path = merge_paths(snk_host_system,
			snk_dirname, xfer_ctrl.snk_rel_path);
		if (src_host_system == SYS_VMS && !preserve_vms_ver_nums) {
			if ((q = strrchr(xfer_ctrl.snk_path, ';')))
				*q = '\0';
		}

		/* Initialize file transfer */
		init_xfer_monitor1();
		retval = (*init_fn[src_host_type][snk_host_type])();
		switch (retval) {
		case -5:
			lost_src_host = True;
			lost_snk_host = True;
			goto lost;
		case -4:
			lost_snk_host = True;
			goto lost;
		case -3:
			lost_src_host = True;
			goto lost;
		case -1:
			sprintf(msg, "Unable to initialize transfer of %s.",
						 xfer_ctrl.head[xfer_ctrl.level]->entry);
			record_warning(msg);
			xfer_ctrl.error = True;
			xfer_ctrl.state = 0;
			xfer_ctrl.nretries = 0;
			ptr = xfer_ctrl.head[xfer_ctrl.level];
			xfer_ctrl.head[xfer_ctrl.level] = ptr->next;
			XtFree(xfer_ctrl.unix_src_rel_path);
			XtFree(xfer_ctrl.unix_snk_rel_path);
			XtFree(xfer_ctrl.src_path);
			XtFree(xfer_ctrl.snk_path);
			XtFree(xfer_ctrl.src_rel_path);
			XtFree(xfer_ctrl.snk_rel_path);
			XtFree(ptr->entry);
			XtFree((char *)ptr);
			return False;
		case 0:
			init_xfer_monitor2();
			start_timer();
			xfer_ctrl.state = 1;
		}

	case 1:   /* State 1: transfer in progress */
		if (xfer_ctrl.abort_requested)
			goto abort;
		retval = (*do_fn[src_host_type][snk_host_type])();
		switch (retval) {
		case -5:
			lost_src_host = True;
			lost_snk_host = True;
			goto lost;
		case -4:
			lost_snk_host = True;
			goto lost;
		case -3:
			lost_src_host = True;
			goto lost;
		case -6:    /* LLNL Archive retry logic */
			if (xfer_ctrl.nretries++ < max_ftp_retries)  {
				xfer_ctrl.state = 0;
				sprintf(msg, "*** Transfer of %s failed.  Will retry.\n",
					xfer_ctrl.head[xfer_ctrl.level]->entry);
				write_log(msg);
				return False;
			}
		case -1:
			sprintf(msg, "Unable to complete transfer of %s.",
						 xfer_ctrl.head[xfer_ctrl.level]->entry);
			record_warning(msg);
			xfer_ctrl.error = True;
			ptr = xfer_ctrl.head[xfer_ctrl.level];
			xfer_ctrl.head[xfer_ctrl.level] = ptr->next;
			XtFree(xfer_ctrl.unix_src_rel_path);
			XtFree(xfer_ctrl.unix_snk_rel_path);
			XtFree(xfer_ctrl.src_path);
			XtFree(xfer_ctrl.snk_path);
			XtFree(xfer_ctrl.src_rel_path);
			XtFree(xfer_ctrl.snk_rel_path);
			XtFree(ptr->entry);
			XtFree((char *)ptr);
			xfer_ctrl.state = 0;
			xfer_ctrl.nretries = 0;
			return False;
		case 0:   /* Transfer complete */
			update_xfer_monitor();
			ftime = stop_timer();
			if (diagnostics >= NORMAL) {
				sprintf(msg, "*** Successfully transferred file:  %s\n",
						xfer_ctrl.src_rel_path);
				write_log(msg);
				print_xfer_stats(ftime, xfer_ctrl.file_len);
			}
			ptr = xfer_ctrl.head[xfer_ctrl.level];
			xfer_ctrl.head[xfer_ctrl.level] = ptr->next;
			XtFree(xfer_ctrl.unix_src_rel_path);
			XtFree(xfer_ctrl.unix_snk_rel_path);
			XtFree(xfer_ctrl.src_path);
			XtFree(xfer_ctrl.snk_path);
			XtFree(xfer_ctrl.src_rel_path);
			XtFree(xfer_ctrl.snk_rel_path);
			XtFree(ptr->entry);
			XtFree((char *)ptr);
			xfer_ctrl.state = 0;
			xfer_ctrl.nretries = 0;
			return False;
		case 1:   /* More to transfer */
			update_xfer_monitor();
			return False;
		}
	}

done:
	
	if (xfer_ctrl.abort_requested)
		hide_abort_dialog();
	else
		hide_xfer_monitor();
	enable_controls(True);
	clear_selected_entries();
	goto sink;

abort:

	retval = (*abort_fn[src_host_type][snk_host_type])();
	switch (retval) {
	case -5:
		lost_src_host = True;
		lost_snk_host = True;
		goto lost;
	case -4:
		lost_snk_host = True;
		goto lost;
	case -3:
		lost_src_host = True;
		goto lost;
	default:
		hide_abort_dialog();
		enable_controls(True);
		clear_selected_entries();
		write_log("*** File transfer(s) aborted.");
		goto sink;
	}

lost:

	if (xfer_ctrl.abort_requested)
		hide_abort_dialog();
	else
		hide_xfer_monitor();
	enable_controls(True);
	clear_selected_entries();
	if (lost_src_host) {
		restore_prev_cursor();
		lost_connection(src_host, snk_dirwin);
		use_busy_cursor();
	}
	if (lost_snk_host) {
		restore_prev_cursor();
		lost_connection(snk_host, snk_dirwin);
		use_busy_cursor();
		clean_up_xfer_ctrl();
		update_dir_controls();
		if (beep_when_ops_done)
			beep();
		restore_prev_cursor();
		return True;
	}

sink:

	/* Update sink directory window */
	if (is_dir_displayed(xfer_ctrl.snk_dirwin->host, xfer_ctrl.snk_dirname,
			&dirwin, &einfo)) {
		if (einfo) {
			if (dirwin != xfer_ctrl.snk_dirwin)  /* Sanity check */
				fatal_error("Bug in cb_xfer_files()");
			indx = xfer_ctrl.snk_dirwin->first_visible_row;
			anchor_rel_path =
				entry_to_rel_path(&xfer_ctrl.snk_dirwin->entries[indx]);
			ref_tree = create_reference_subtree(einfo);
			collapse_subtree(&einfo, False);
			retval = re_expand_subtree(&einfo, ref_tree);
			release_reference_subtree(ref_tree);
			switch (retval) {
			case -1:
				record_and_alert(msg1, snk_dirwin->w_shell);
				XtFree(anchor_rel_path);
				goto fin;
			case -3:
				lost_connection(snk_host, snk_dirwin);
				XtFree(anchor_rel_path);
				goto fin;
			}
			calc_dirwin_geometry(xfer_ctrl.snk_dirwin);
			set_first_visible_entry(xfer_ctrl.snk_dirwin, anchor_rel_path);
			XtFree(anchor_rel_path);
		} else {
			switch (display_dir(snk_host, dirwin, snk_dirname, True, False)) {
			case -1:
				record_and_alert(msg1, snk_dirwin->w_shell);
				break;
			case -3:
				lost_connection(snk_host, snk_dirwin);
			}
		}
	}

fin:

	/* Finish up */
	if (beep_when_ops_done)
		beep();
	clean_up_xfer_ctrl();
	update_dir_controls();
	restore_prev_cursor();
	return True;
}


/*
 * extract_file_len - Extracts the file length usually returned in a "150"
 *                    ftp reply.  This routine looks for a pattern of the
 *                    form "(xxx bytes)", "(xxx)", or "(xxxk) near the end
 *                    of the reply, where the "xxx" represent digits.  If
 *                    this pattern is found, the file length is returned,
 *                    otherwise -1 is returned.  "reply" is the "150"
 *                    ftp reply to be scanned.
 */
long
extract_file_len(reply)
char *reply;
{
    static char *pat1 = " bytes)";
    int lenpat1 = strlen(pat1);
	static char *pat2 = ")";
	int lenpat2 = strlen(pat2);
	static char *pat3 = "k)";
	int lenpat3 = strlen(pat3);
	char *ptr;
	long len;

    if ((ptr = strrchr(reply, '(')) && isdigit(*++ptr)) {
        len = 0;
        while (*ptr && isdigit(*ptr))
            len = (len*10)+(*ptr++-'0');
        if ((int)(strlen(reply)-strlen(ptr)) >= lenpat1)
            if (strncmp(ptr, pat1, lenpat1) == 0)
                return len;
        if ((int)(strlen(reply)-strlen(ptr)) >= lenpat2)
            if (strncmp(ptr, pat2, lenpat2) == 0)
                return len;
        if ((int)(strlen(reply)-strlen(ptr)) >= lenpat3)
            if (strncmp(ptr, pat3, lenpat3) == 0)
                return len*1024;
    }

	return -1;
}


/*
 * start_timer - Starts the microsecond timer.  Needs to be called at
 *               start of interval to be timed.  Call stop_timer() at
 *               end of interval.
 *
 */
start_timer()
{
    gettimeofday(&start_time, NULL);
}


/*
 * stop_timer - Returns the number of microseconds that have elapsed
 *              since start_timer() was last called.  Undefined results
 *              if start_timer() is not called first.
 */
double
stop_timer()
{
    double fstart_time;
    double fstop_time;

    gettimeofday(&stop_time, NULL);
    fstart_time = (double)start_time.tv_sec+(double)start_time.tv_usec/1000000.;
    fstop_time = (double)stop_time.tv_sec+(double)stop_time.tv_usec/1000000.;
    return fstop_time-fstart_time;
}


/*
 * print_xfer_stats - Prints statistics about a file transfer.  "ftime"
 *                    is the number of seconds the transfer took.  "nbytes"
 *                    is the number of bytes transferred.  Set "nbytes"
 *                    to -1 if file length not known.
 */
print_xfer_stats(ftime, nbytes)
double ftime;
long nbytes;
{
	char msg[120];

	write_log("      ");
	if (nbytes == -1)
		write_log("?");
	else {
		sprintf(msg, "%ld", nbytes);
		write_log(msg);
	}
	sprintf(msg, " bytes transferred in %3.1f seconds", ftime);
	write_log(msg);
	if (nbytes != -1) {
		if (ftime <= 0)
			write_log(" (Very Fast)");
		else {
			sprintf(msg, " (%3.1f Kbytes/s)", (double)nbytes/(ftime*1000.));
			write_log(msg);
		}
	}
	write_log("\n");
}


/*
 * clean_up_xfer_ctrl - Cleans up the control structure used by the file
 *                      transfer work proc.
 */
clean_up_xfer_ctrl()
{
	int i;

	XtFree(xfer_ctrl.unix_src_rel_path);
	xfer_ctrl.unix_src_rel_path = NULL;

	XtFree(xfer_ctrl.unix_snk_rel_path);
	xfer_ctrl.unix_snk_rel_path = NULL;

	XtFree(xfer_ctrl.src_rel_path);
	xfer_ctrl.src_rel_path = NULL;

	XtFree(xfer_ctrl.snk_rel_path);
	xfer_ctrl.snk_rel_path = NULL;

	XtFree(xfer_ctrl.src_path);
	xfer_ctrl.src_path = NULL;

	XtFree(xfer_ctrl.snk_path);
	xfer_ctrl.snk_path = NULL;

	XtFree(xfer_ctrl.src_dirname);
	xfer_ctrl.src_dirname = NULL;

	XtFree(xfer_ctrl.snk_dirname);
	xfer_ctrl.snk_dirname = NULL;

	for(i=0; i<=xfer_ctrl.level; i++)
		release_linked_list(&xfer_ctrl.head[i]);
}

